#!/usr/bin/python3

import sys
sys.path.insert(0, "../../../Python")

import rumd
# it is essential that there is no doubt about which version is actually run!
print("File used to import the rumd module", rumd.__file__)

from rumd.Simulation import Simulation
from rumd.Autotune import Autotune

def RunMol_Autotune():

    sim = Simulation("startN256.xyz.gz", verbose=False)

    # Create LJ pair-potential
    potential = rumd.Pot_LJ_12_6(cutoff_method=rumd.ShiftedPotential)
    potential.SetParams(0, 0, 1.00, 1.00, 2.5)
    sim.AddPotential(potential)

    # integrator
    itg = rumd.IntegratorNVT(timeStep=0.004, targetTemperature=0.5, thermostatRelaxationTime=0.2)
    sim.SetIntegrator(itg)

    # Molecule stuff
    sim.ReadMoleculeData("molN256.top")

    pot_harm = rumd.BondHarmonic()
    pot_harm.SetParams(0, 1.0, 200.0, exclude=True)
    sim.AddPotential(pot_harm)
    

    at = Autotune(ignore_file=True)
    at.test_mode = True
    at.Tune(sim)


if __name__ == "__main__":

    RunMol_Autotune()
