#!/usr/bin/python3

import sys
sys.path.insert(0, "../../../Python")

import rumd
# it is essential that there is no doubt about which version is actually run!
print("File used to import the rumd module", rumd.__file__)

from rumd.Simulation import Simulation
from rumd.Autotune import Autotune

def RunSLLOD_Autotune():
    """ Use autotuner in test mode to detect problems with SLLOD"""
    # use a start configuration which already has a significant amount of
    # Lees-Edwards shift, so incorrect handling of Lees-Edwards BC will
    # quickly give wrong results
    sim = Simulation("shiftedN1000.xyz.gz")

    # integrator
    itg = rumd.IntegratorSLLOD(timeStep=0.005, strainRate=0.01)

    # Create IPL pair-potential
    potential = rumd.Pot_IPL_12(cutoff_method=rumd.ShiftedPotential)
    # i  j   Sig   Eps  Rcut (in units of Sigma_ij)
    potential.SetParams(0, 0, 1.00, 1.00, 2.5)


    sim.SetIntegrator(itg)
    sim.AddPotential(potential)

    potential.SetNB_Method("none") # most likely to work--use as reference

    at = Autotune(ignore_file=True)
    at.test_mode = True
    at.Tune(sim)


if __name__ == "__main__":

    RunSLLOD_Autotune()
