#!/usr/bin/python3

import sys
import unittest
sys.path.insert(0, "../../../Python")

import rumd
from rumd.Simulation import Simulation


class TestShearBoundarySmall(unittest.TestCase):
    """ Class for testing shear for small systems"""
    def setUp(self):
        self.sim = Simulation("traj0076_0038_srt_vel.xyz.gz")


        # Create LJ pair-potential
        self.potential = rumd.Pot_LJ_12_6(cutoff_method=rumd.ShiftedPotential)
        self.potential.SetParams(i=0, j=0, Epsilon=1.0, Sigma=1.0, Rcut=2.5)
        self.potential.SetNbListSkin(0)

        self.sim.AddPotential(self.potential)

        # Molecule stuff
        self.sim.ReadMoleculeData("start_srt.top")
        pot_harm = rumd.BondHarmonic()
        pot_harm.SetParams(bond_type=0, bond_length=1.0, stiffness=100.0, exclude=True)
        self.sim.AddPotential(pot_harm)

        self.reference_pe = -0.32189828

    def test_unsorted(self):
        """ test where the potential energy is calculated with non-sorting based
        neighbor-list generation"""
        self.sim.sample.CalcF()
        pe = self.sim.sample.GetPotentialEnergy()/self.sim.GetNumberOfParticles()
        #print "pe",pe
        self.assertLess(abs(pe-self.reference_pe), 1.e-6)

    def test_sorted(self):
        """ test where the potential energy is calculated with sorting based
        neighbor-list generation"""
        self.sim.sample.SortParticles()
        self.sim.sample.CalcF()
        pe = self.sim.sample.GetPotentialEnergy()/self.sim.GetNumberOfParticles()

        #print "pe",pe
        self.assertLess(abs(pe-self.reference_pe), 1.e-6)

if __name__ == '__main__':
    unittest.main()
